@@ -15,7 +15,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <syslog.h>
 #include <fcntl.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -29,13 +28,13 @@
 static void
 explain(void)
 {
-	fprintf(stderr, "Usage: mirred <DIRECTION> <ACTION> [index INDEX] <dev DEVICENAME>\n");
-	fprintf(stderr, "where:\n");
-	fprintf(stderr, "\tDIRECTION := <ingress | egress>\n");
-	fprintf(stderr, "\tACTION := <mirror | redirect>\n");
-	fprintf(stderr, "\tINDEX  is the specific policy instance id\n");
-	fprintf(stderr, "\tDEVICENAME is the devicename\n");
-
+	fprintf(stderr,
+		"Usage: mirred <DIRECTION> <ACTION> [index INDEX] <dev DEVICENAME>\n"
+		"where:\n"
+		"\tDIRECTION := <ingress | egress>\n"
+		"\tACTION := <mirror | redirect>\n"
+		"\tINDEX  is the specific policy instance id\n"
+		"\tDEVICENAME is the devicename\n");
 }
 
 static void
@@ -61,6 +60,34 @@ static const char *mirred_n2a(int action)
 	}
 }
 
+static const char *mirred_direction(int action)
+{
+	switch (action) {
+	case TCA_EGRESS_REDIR:
+	case TCA_EGRESS_MIRROR:
+		return "egress";
+	case TCA_INGRESS_REDIR:
+	case TCA_INGRESS_MIRROR:
+		return "ingress";
+	default:
+		return "unknown";
+	}
+}
+
+static const char *mirred_action(int action)
+{
+	switch (action) {
+	case TCA_EGRESS_REDIR:
+	case TCA_INGRESS_REDIR:
+		return "redirect";
+	case TCA_EGRESS_MIRROR:
+	case TCA_INGRESS_MIRROR:
+		return "mirror";
+	default:
+		return "unknown";
+	}
+}
+
 static int
 parse_direction(struct action_util *a, int *argc_p, char ***argv_p,
 		int tca_id, struct nlmsghdr *n)
@@ -71,7 +98,7 @@ parse_direction(struct action_util *a, int *argc_p, char ***argv_p,
 	int ok = 0, iok = 0, mirror = 0, redir = 0, ingress = 0, egress = 0;
 	struct tc_mirred p = {};
 	struct rtattr *tail;
-	char d[16] = {};
+	char d[IFNAMSIZ] = {};
 
 	while (argc > 0) {
 
@@ -80,7 +107,8 @@ parse_direction(struct action_util *a, int *argc_p, char ***argv_p,
 		} else if (!egress && matches(*argv, "egress") == 0) {
 			egress = 1;
 			if (ingress) {
-				fprintf(stderr, "Can't have both egress and ingress\n");
+				fprintf(stderr,
+					"Can't have both egress and ingress\n");
 				return -1;
 			}
 			NEXT_ARG();
@@ -89,7 +117,8 @@ parse_direction(struct action_util *a, int *argc_p, char ***argv_p,
 		} else if (!ingress && matches(*argv, "ingress") == 0) {
 			ingress = 1;
 			if (egress) {
-				fprintf(stderr, "Can't have both ingress and egress\n");
+				fprintf(stderr,
+					"Can't have both ingress and egress\n");
 				return -1;
 			}
 			NEXT_ARG();
@@ -110,30 +139,35 @@ parse_direction(struct action_util *a, int *argc_p, char ***argv_p,
 					break;
 				}
 			} else if (!ok) {
-				fprintf(stderr, "was expecting egress or ingress (%s)\n", *argv);
+				fprintf(stderr,
+					"was expecting egress or ingress (%s)\n",
+					*argv);
 				break;
 
 			} else if (!mirror && matches(*argv, "mirror") == 0) {
 				mirror = 1;
 				if (redir) {
-					fprintf(stderr, "Can't have both mirror and redir\n");
+					fprintf(stderr,
+						"Can't have both mirror and redir\n");
 					return -1;
 				}
 				p.eaction = egress ? TCA_EGRESS_MIRROR :
-					    TCA_INGRESS_MIRROR;
+					TCA_INGRESS_MIRROR;
 				p.action = TC_ACT_PIPE;
 				ok++;
 			} else if (!redir && matches(*argv, "redirect") == 0) {
 				redir = 1;
 				if (mirror) {
-					fprintf(stderr, "Can't have both mirror and redir\n");
+					fprintf(stderr,
+						"Can't have both mirror and redir\n");
 					return -1;
 				}
 				p.eaction = egress ? TCA_EGRESS_REDIR :
-					    TCA_INGRESS_REDIR;
+					TCA_INGRESS_REDIR;
 				p.action = TC_ACT_STOLEN;
 				ok++;
-			} else if ((redir || mirror) && matches(*argv, "dev") == 0) {
+			} else if ((redir || mirror) &&
+				   matches(*argv, "dev") == 0) {
 				NEXT_ARG();
 				if (strlen(d))
 					duparg("dev", *argv);
@@ -150,18 +184,16 @@ parse_direction(struct action_util *a, int *argc_p, char ***argv_p,
 		NEXT_ARG();
 	}
 
-	if (!ok && !iok) {
+	if (!ok && !iok)
 		return -1;
-	}
-
-
 
 	if (d[0])  {
 		int idx;
 
 		ll_init_map(&rth);
 
-		if ((idx = ll_name_to_index(d)) == 0) {
+		idx = ll_name_to_index(d);
+		if (idx == 0) {
 			fprintf(stderr, "Cannot find device \"%s\"\n", d);
 			return -1;
 		}
@@ -170,23 +202,26 @@ parse_direction(struct action_util *a, int *argc_p, char ***argv_p,
 	}
 
 
-	if (p.eaction == TCA_EGRESS_MIRROR || p.eaction == TCA_INGRESS_MIRROR)
+	if (p.eaction == TCA_EGRESS_MIRROR || p.eaction == TCA_INGRESS_MIRROR) {
 		parse_action_control(&argc, &argv, &p.action, false);
+		NEXT_ARG_FWD();
+	}
 
 	if (argc) {
 		if (iok && matches(*argv, "index") == 0) {
 			fprintf(stderr, "mirred: Illegal double index\n");
 			return -1;
-		} else {
-			if (matches(*argv, "index") == 0) {
-				NEXT_ARG();
-				if (get_u32(&p.index, *argv, 10)) {
-					fprintf(stderr, "mirred: Illegal \"index\"\n");
-					return -1;
-				}
-				argc--;
-				argv++;
+		}
+
+		if (matches(*argv, "index") == 0) {
+			NEXT_ARG();
+			if (get_u32(&p.index, *argv, 10)) {
+				fprintf(stderr,
+					"mirred: Illegal \"index\"\n");
+				return -1;
 			}
+			argc--;
+			argv++;
 		}
 	}
 
@@ -243,7 +278,7 @@ parse_mirred(struct action_util *a, int *argc_p, char ***argv_p,
 }
 
 static int
-print_mirred(struct action_util *au, FILE * f, struct rtattr *arg)
+print_mirred(struct action_util *au, FILE *f, struct rtattr *arg)
 {
 	struct tc_mirred *p;
 	struct rtattr *tb[TCA_MIRRED_MAX + 1];
@@ -255,27 +290,29 @@ print_mirred(struct action_util *au, FILE * f, struct rtattr *arg)
 	parse_rtattr_nested(tb, TCA_MIRRED_MAX, arg);
 
 	if (tb[TCA_MIRRED_PARMS] == NULL) {
-		fprintf(f, "[NULL mirred parameters]");
+		print_string(PRINT_FP, NULL, "%s", "[NULL mirred parameters]");
 		return -1;
 	}
 	p = RTA_DATA(tb[TCA_MIRRED_PARMS]);
 
-	/*
-	ll_init_map(&rth);
-	*/
-
-
-	if ((dev = ll_index_to_name(p->ifindex)) == 0) {
+	dev = ll_index_to_name(p->ifindex);
+	if (dev == 0) {
 		fprintf(stderr, "Cannot find device %d\n", p->ifindex);
 		return -1;
 	}
 
-	fprintf(f, "mirred (%s to device %s)", mirred_n2a(p->eaction), dev);
+	print_string(PRINT_ANY, "kind", "%s ", "mirred");
+	print_string(PRINT_FP, NULL, "(%s", mirred_n2a(p->eaction));
+	print_string(PRINT_JSON, "mirred_action", NULL,
+		     mirred_action(p->eaction));
+	print_string(PRINT_JSON, "direction", NULL,
+		     mirred_direction(p->eaction));
+	print_string(PRINT_ANY, "to_dev", " to device %s)", dev);
 	print_action_control(f, " ", p->action, "");
 
-	fprintf(f, "\n ");
-	fprintf(f, "\tindex %u ref %d bind %d", p->index, p->refcnt,
-		p->bindcnt);
+	print_uint(PRINT_ANY, "index", "\n \tindex %u", p->index);
+	print_int(PRINT_ANY, "ref", " ref %d", p->refcnt);
+	print_int(PRINT_ANY, "bind", " bind %d", p->bindcnt);
 
 	if (show_stats) {
 		if (tb[TCA_MIRRED_TM]) {
@@ -284,7 +321,7 @@ print_mirred(struct action_util *au, FILE * f, struct rtattr *arg)
 			print_tm(f, tm);
 		}
 	}
-	fprintf(f, "\n ");
+	print_string(PRINT_FP, NULL, "%s", "\n ");
 	return 0;
 }
 
